# Packages needed
library(VGAM) # To simulate the Pareto distribution
library(ggplot2) # To make the gradient plots of Fig. 2
library(grid) # For the function unit() used in Fig. 2
library(gridExtra) # To join the plots of Fig. 2 into one multi-panelled figure

#######################################################################################
# Simulate a CCRW
# Reynolds (2012) simmulates the CCRW as follow:
# "CCRW with 1000 steps are, for example, frequently mistaken 
# for being LW when step lengths are drawn at random from two
# continuous exponential distributions, one for relatively short
# movements with mean length 15 (arbitrary length units) used
# with a 0.9 probability, the other for relatively long movements
# with mean length 74 used with a 0.1 probability (Fig. 1a).
# [...] Ten-thousand different CCRW models were examined with 
# mean short and long step lengths ranging between 1 and 100
# and with move-type selection probabilities ranging between 0.1
# and 0.9. Each of these models was used to simulate a single
# CCRW with 10 000 steps;"
# This function simmulates a CCRW as defined by Reynolds (2012).
# It uses 2 exponential distributions to sample the step lengths.
# The exponential distributions differ in their mean step length: sI and sE.
# Each exponential distribution is associated with one of the two behavioural phases.
# The transition between the behavioural phases is achieved via a bernoulli process.
# Reynolds (2012) does not specify a minimum step length for his simulations of the CCRW.
# Thus, we assume that the minimum step length is 0.
# Note that minimum step length refers to the minimum simulated step length,
# which woud mean the minimum movement that an animal can produce.
# This differs from the cut-off, a,
# which represent the start of the tail of the power-law distribution.

simmCCRW <- function(n, pI, sI, sE){
  # Inputs are:
  #   n is the number of steps,
  #   pI is the probability of being in the intensive behaviour, 
  #   thus the probability of being in the extensive behaviour is pE = 1 - pI,
  #   sI is the mean step length for the intensive behaviour, and
  #   sE is the mean step length for the extensive behaviour.
  
  # Sample the time-series of behavioural phases using a bernoulli process
  # Because we use the value as an index of the behavioural phase
  # and R start indexing at 1, we add 1 to the results from the rbinom().
  # rbinom(n, size=1, prob) returns 0 for failure and 1 for success.
  # Thus, rbinom(n, size=1, prob=pE)+1 returns 1 for intensive 
  # and 2 for extensive behvaiour.
  b <- rbinom(n, 1, (1-pI)) + 1
  
  # rexp() samples from an exponential distribution.
  # It uses the rate (i.e. lambda) rather than the mean step length.
  # The rate is the inverse of the mean step lengh.
  l <- 1/c(sI,sE)
  
  # Sample the step length using the lambda for the behavioral phase selected.
  SL <- rexp(n,l[b])
    
  return(SL)
}

#######################################################################################
# Simulate a discrete TLW
# Reynolds (2012) states:
# "Here and throughout, TLW were simulated by drawing
# step lengths at random from a Pareto distribution: 
# p(l) = (mu - 1)*a^(mu-1)*l^(-mu) for l >= a and p(l) = 0
# for l < a where a = 1 arbitrary length units." 
# However, his Fortran Code found in the Supplementary material
# Power-spectrum.for states:
#         
#         c SET UP RANDOM NUMBER GENERATOR FOR LW 
#         mu=2.0d0
#         Ptot=0.0d0
#         
#         c Set up reference vector          
#         do 5 ix=1,100000     ! Define distribution     
#         x=1.0d0*ix             
#         P(ix)=1.0d0/(x**mu)
#         Ptot=Ptot+P(ix)     
#         5   continue
#         
#         c normalize pdf
#         do 10 ix=1,100000
#         P(ix)=P(ix)/Ptot
#         10   continue
#         
# Therefore the pdf created is for a discrete truncated LW,
# not a continuous non-truncated LW as stated in his main text.
# His discrete TLW allows only for integer values between 1 and 100000

simmDTLW <- function(n, mu, xmin, xmax){
  # Inputs are:
  #   n is the number of steps,
  #   mu is the shape parameter of the Pareto distribution,
  #   xmin is location parameter (the minimum step length made by the animal), and
  #   xmax is the maximum step length or the truncation point.
  
  # Sampling from the Pareto as described by Reynolds (2012) Supplementary material 
#   P <- numeric((xmax-xmin+1))     # double precision P(100000),R(140005)
#   Ptot <- 0                       # Ptot=0.0d0
#   # c Set up reference vector 
#   for(ix in xmin:xmax){           # do 5 ix=1,100000 ! Define distribution   
#     x <- ix                       #   x=1.0d0*ix          
#     P[ix] <- 1/(x^mu)             #   P(ix)=1.0d0/(x**mu)
#     Ptot <- Ptot + P[ix]          #   Ptot=Ptot+P(ix)     
#   }                               # 5 continue
#   # c normalize pdf
#   for(ix in xmin:xmax){           # do 10 ix=1,100000
#     P[ix] <-  P[ix]/Ptot          #   P(ix)=P(ix)/Ptot
#   }                               # 10 continue  
  
  # More efficient way to write the two previous loops.
  P <- 1/((xmin:xmax)^mu)
  P <- P/sum(P)
  
  SL <- sample(xmin:xmax, n, prob=P, replace=TRUE) # k=g05eyf(r,nr)
  
  return(SL)
}

#######################################################################################
# Simulate a continuous TLW
# As explained in our main manuscript, there is a difference between the power spectra
# resulting from the a turning point time-series of a discrete TLW 
# and that of a continuous TLW.
# We simulate the continuous TLW to be able to compare it to discrete TLW.
# This functions simmulates a continuous TLW,
# which is much closer to LW described in the main text from Reynolds (2012),
# than the one described in Reynolds (2012) supplementary material.
# The only difference between the continuous LW we are simulating 
# and the one described in the main text of Reynolds (2012)
# is that we are using the truncated version of the Pareto 
# rather than the non-truncated (pure) Pareto.
# We chose the continuous truncated Pareto to be consistent 
# with the truncated discrete Pareto that Reynolds (2012) use in his code.
# The package VGAM has a function to sample from the Pareto distribution
# It uses the Pareto definition of Evans et al. (2000) (see ?rpareto)
# Note that mu is the equivalent of c+1 
# and the pareto functions in R use c rather than mu.

simmTLW <- function(n, mu, xmin, xmax){
  # Inputs are:
  #   n is the number of steps,
  #   mu is the shape parameter of the Pareto distribution,
  #   xmin is location parameter (the minimum step length made by the animal), and
  #   xmax is the maximum step length or the truncation point.
  
  SL <- rtpareto(n, xmin, xmax, (mu-1))
  
  return(SL)
}

#######################################################################################
# Calculate the Akaike weight of the LW and the MLE mu
# Reynolds (2012) calculates the Akaike weights as follow:
# "Here, following Edwards (2007) I re-examine this assertion and use Akaike weights, w,
# to test whether simulation data for CCRW provides more evidence for step lengths, l,
# coming from power-law distributions
# P_1(l) = (mu - 1)*a^(mu-1) * l^(-mu)  	l >= a
# or from exponential distributions
# P_2(l) = lambda*exp(-lambda*(l-a)) 			l >= a
# where a is the start of the tail of the distribution of step lengths and lambda is 
# the exponential decay constant. [...] The Akaike weight, w_p, for the power-law
# distribution can be considered as the weight of evidence in favor of the 
# power-law distribution being the best model for the data, out of the considered models.
# The Akaike weight for a power-law can vary from 0 (no support) to 1 (complete support)
# (Edwards et al. 2007). [...] The maximum likelihood estimates for mu = 2." 
# Although Reynolds (2012) does not mention specifically how he calculates the MLE,
# we are assuming he is using the method of Edwards et al. (2007).
# This is consistent with the code of in his supplementary material AIC-weigths.for.
# Note that he does not specify what he uses for the minimum step length, a.
# We are assuming that a is known and specified for two reasons.
# First, because in Reynolds (2012) supplementary material AIC-weights.for he states:
#           c AIC test parameters
#           
#           c lower cut-off
#           alim=10.0d0          
# Second, because setting the cut-off much greater than the minimum simulated step length (xmin)
# is the only way we were able to reproduce Fig. 1 from Reynolds (2012).

wLW <- function(SL, a){
  # Inputs are:
  #   SL is the simulated time-series of step lengths, and
  #   a is the cut-off value (start of the tail of the power-law).
  
  # Sample size
  n <- length(SL)
  
  # MLE eqn. from Edwards et al. (2007) for mu of the Pareto distribution
  mu <- 1 - n/(n*log(a)-sum(log(SL)))
  # MLE eqn. from Edwards et al. (2007) for lambda of a single exponential distribution
  lambda <- 1/(mean(SL)-a)
  
  # Calculating the AIC
  # Note that since we are assuming that a is known,
  # only mu is estimated for Pareto and only lambda is estimated for the exponential.
  # Log-likelihood from Edwards et al. (2007) for the Pareto    
  LLW <- n*log(mu-1) + n*(mu-1)*log(a) - mu*sum(log(SL))
  # AIC for LW
  AICLW <- -2*LLW +2
  # Log-likelihood from Edwards et al. (2007) for the single exponential    
  LEXP <- n*log(lambda) + n*lambda*a - lambda*sum(SL)
  # AIC for single exponential
  AICEXP <- -2*LEXP + 2
  
  # Calculating the AKaike weight for the LW
  # Find the minimum AIC
  AICmin <- min(AICLW, AICEXP)
  # Calculate delat AIC
  dLW <- AICLW - AICmin
  dEXP <- AICEXP - AICmin
  # Akaike weigth for LW
  w <- exp(-0.5*dLW)/(exp(-0.5*dLW) + exp(-0.5*dEXP)) 
  
  return(c(mu=mu,w=w))
}

#######################################################################################
# Apply the G-test for the LW
# Reynolds (2012) calculates the G-test as follow:
# "The application of goodness-of-fit tests performed using the G test (likelihood-ratio
# test) (Sokal and Rohlf 1995) confirmed that data with wp ~= 1 and wp  ~= 0 were 
# consistent with power-law and exponential distributions, i.e., are consistent with LW
# and simple correlated random walks. These fits are statistically significant (df = 1000,
# P < 0.05). Similar results were found for other CCRW models."
# Reynolds (2012) does not provide the code for the G-test in his Supplementary material.
# Thus we based the G-test on Sokal and Rohlf (1995).
# To get the degrees of freedom of Reynolds (2012), df=1000, 
# we need to set the number of bins to 1002.
# This is because according to Sokal and Rohlf (1995) 
# the degrees of freedom are calulated as follow:
# df = nB - nP - 1,
# where nB is the number of bins and nP is the number of parameter estimated.
# For the LW, we are only estimating mu, so df = nB - 2, thus 1000 = nB -2 and nB = 1002.
# According to Sokal and Rohlf (1995) the G-test will be negatively biased
# if the expected values of certain bins are small.
# To limit the number of bins with small size 
# we create bins qith equal expected values of n/nB. 
# When n is 10000 and the bins are divided to contain equal number of expected values,
# the expected value of each bin is 10000/1002 = 9.98
# Thus using 1002 bins is adequate for 10000 steps 
# but will become inadaquate for smaller sample size.
# Also note that Sokal and Rohlf (1995) recommend using the William's correction.

GTest <- function(SL, mu, a, nB){
  # Inputs are:
  #   SL is the simulated time-series of step lengths, 
  #   mu is the shape parameter of the Pareto,
  #   a is the cut-off value (start of the tail of the power-law), and
  #   nB is the number of bins used in the G-test.
  
  # Using the quantile function for the continuous non-truncated Pareto distribution,
  # which is the same distribution as the one used for the Akaike weigths and MLE,
  # we create the specified amount of bins, nB.
  # Note that the lower limit of the first bin is the cut-off, a, and
  # the upper limit of the last bin is Inf,
  # beacuse this is a non-truncated Pareto distribution.
  bin <- c(a,qpareto(1/nB*1:(nB-1), a, (mu-1)), Inf)
  # Using the quantile function for the Pareto (from the package VGAM), 
  # results in each bin having the same expected number of observations.
  fE <- rep(length(SL)/nB, nB)  
  # Counting the number of observed step lengths in each bin.
  fO <- hist(SL, breaks=bin, plot=FALSE)$counts
  
  # Bins that have 0 observations cause problems.
  # We lump them to non-zero bins and change the df accordingly.
  for(i in sort(which(fO==0),decreasing=TRUE)){
    fO[i-1] <- fO[i] + fO[i-1]
    fO <- fO[-i]
    fE[i-1] <- fE[i] + fE[i-1]
    fE <- fE[-i]
  }
  
  # Calculating G
  GVal <- fO*log(fO/fE)
  GVal <- 2*sum(GVal)
  
  # Calculating the degrees of freedom
  dfGT <- length(fE) - 1 - 1
  
  # Williams correction according to Sokal and Rohlf (1995)
  # q=1+(a^2-1)/6* n*dfGT, where a is number of category
  wc <- 1 + (nB^2 -1)/(6 * length(SL) * dfGT)
  
  # Calculating the p-value for the G-test
  p <- pchisq(GVal/wc,dfGT,lower.tail=FALSE)
  return(p)
}

#######################################################################################
# Apply the power-spectra method
# Reynolds (2012) calculates the power-spectra in two main steps.
#
# Step 1. He creates a u(t) time series:
# "In both models a new step commences each and every time the walker changes its direction
# of travel, i.e., each and every time it "turns." [...] A turn can, for instance, be taken
# to can have arisen when the angle between two movement segments joining three successive
# positional fixes is less than a critical angle (Reynolds et al. 2007a); a prescription that
# is closely analogous to that advocated by Turchin (1998) and Sims et al. (2007). The 
# significant turns define a time series u(t). If significant turns occurred at times 
# t = 3 delta t, 5 delta t, 6 delta t, ..., (N - 2) delta t, N delta t where delta t is the
# time interval between consecutive positional fixes and N is the number of steps, then the
# entries in the time series u(t) would be 0, 0, 1, 0, 1, 1, . . . , 1, 0, 1."

# Animals do not make their turn at regular times intervals and
# the distribution used for the CCRW produce step lengths with continuous real values.
# Thus, the turns do not happen at regular delta t intervals. 
# To create u(t) from our sampled times of turn, 
# we first take the cummulative sums of the times.
# These cummulative sums represent the time at which the turns are made.
# Second, because the turn will occur in between t and t + delat t,
# we associate the turn to the beginning of the delta t.  
# For example, if we assume a delta t of 1 unit and the turn occurred at 6.7,
# we place the turn at t=6. 
# Third, we create a u(t) with 0 for t when there is no turn 
# and 1 for t when there is a turn.
# The u(t) time series length will depend on the model, 
# for example with LW with low mu creating extremely long u(t) series.
# To make a comparison between u(t) series of the same length 
# we fixed the length of u(t)
# Although Reynolds (2012) does not mention whether u(t) lengh is fixed,
# they are a few clues that he does so.
# In the caption of Fig. 2 of Reynolds (2012) :
# "The ensemble-averaged power spectrum S(f) (where f is the frequency) of the
# time series u(t) of turning points (where u is the time series and t is time)
# in 50 LW with mu=2 and in 50 CCRW. Each walk is 1000r long."

# To be able to directly average the power-spectrum S(f),
# you need the u(t) to be of the same length 
# because you want the frequencies to be the same. 
# The frequencies of the power spectra depends on the length of the u(t). 
# They are defined as:
# f = 0, 1/T, 2/T, ..., (N-1)/T,
# where N is the number of steps in u(t) and T is the total time span.
# Thus, by limiting u(t) to a specify length,
# one can average the power-spectra and
# remove the potential for memory errors.
# Although we could not find a clear statement of the length of u(t) in Reynolds (2012),
# he specify that each walk is 1000r long. 
# r is not defined, but we think it is the length of u(t).
# This is confirmed by a look at his Fortran code:
#           c Simulate LW or CCRW and calculate spectal density
#           n=1000
#           nreal=100
#           do 12 ireal=1,nreal
#             do 15 i=0,n
#               u(i)=0.0d0
#             15  continue
#             i=0
#             do 20 while(i.lt.n)
#               c update timeseries  
#               u(i)=1
#               c LW	     
#               c k=g05eyf(r,nr)
#               c	i=i+k
#               c CCRW
#               pd=g05caf(pd) 
#               if(pd.lt.0.1d0) then
#                 s=g05dbf(74.0d0)
#               else
#                 s=g05dbf(15.0d0)
#               end if
#               k=dint(s)+1
#               i=i+k	      
#             20 continue
#
# This code indicate that an u vector of length n (n=1000) is fills with 1
# everytime there is a significant turn.
# So we create a u(t) time series of specified length ul

# Step 2. Once the u(t) is created Reynolds (2012) applies the power-spectra:
# "The power-spectra of u(t) is the square of the magnitude of the Fourier
# transform of u(t):
# S(f) = |1/sqrt(2*pi) * sum_{k=0}^N (u(t)*exp(-i*2*pi*f*t))|^2
# = 1/(2*pi) * F(f) * F{*}(f)       (1)
# where t=k delta t, is the time at which the kth significant turn was made, 
# S is the power spectra, f is frequency, F(f) is the discrete Fourier transform
# of u(t), and F*(f) is its complex conjugate."
# We use R fft() function to calculate the fast discrete fourier transform

ps <- function(SL, ul){
  # Inputs are:
  #   SL is the simulated time-series of step lengths, and
  #   ul is the length of the u(t) time series.
  
  # Step 1. Create the u(t) time-series
  # A) Identify the times of significant turns by taking the cummulative sums.
  # B) Associate the turns to the beginning of the delta t.
  # Note that rounding instead of taking the floor
  # does not change our conclusion regarding the difference between the DTLW and TLW.
  # i <- round(cumsum(SL))
  i <- floor(cumsum(SL))
  # C) Create an empty u(t) of length ul, 
  u <- integer(ul)
  # D) Place a turn (i.e. 1) at each time their was a turn 
  # and only during the time span of u(t)
  u[i[i<=ul]] <- as.integer(1)
  
  # Make sure that there is enough step lengths in SL to create a u(t) of length ul.
  # If not it gives a warning and make the u(t) the size expected by SL.
  if(i[length(i)]<ul){
    warning(paste("Not enough step lengths to create a u(t) time series of length ", ul))
    ul <- i[length(i)]
    u <- u[1:ul]
  }
  
  # Step 2. Calculate the power spectra of u(t)
  # The function fft() calculates the fast discrete Fourier transform
  # The function Conj() takes the complex conjugate
  S <- fft(u)
  S <- Re((S*Conj(S))/(2*pi))
  # f(k) = (k-1)/T,   k = 1, 2, ..., N 
  # since our delta_t is 1 N=T=ul
  f <- ((1:ul)-1)/ul
  # Get all the nonzero frequencies and associate with S(f)
  S <- cbind(f,S)[-1,]
    
  return(S)
}

#######################################################################################
# Coarse estimate of beta
# The important parameter of the power spectra is the beta
# From Reynolds (2012):
# "The power-spectra for LW with mu = 2 and of length 1000 a exhibit power-law scaling,
# S(f) ~ f^{-beta} with power-law exponent beta ~= 0.7 over an extended range of scales (Fig. 2).
# It can be shown that beta approaches 3 -mu asymptotically for sufficiently long LW (Viswanathan
# et al. 1996). The "1/f" noise is indicative of scale invariance, a hallmark characteristic of
# LW (and truncated LW) and is quite unlike the noise that characterizes CCRW that is always 
# close to being "white" (beta ~= 0.0) (Fig. 2)."
# Reynolds (2012) does not specify how he calculates the beta from the power spectra values. 
# Although the method presented below has known biases, 
# especially if the power spectra is not linear,
# we fit a regression for log(S(f)) ~ log(f) and
# use the negative of the slope as the beta value.

getB <- function(S){
  # Input is:
  #   S is the power spectra results from ps(), it has the frequencies and the spectrum. 
  B <- -lm(log(S[,2])~log(S[,1]))[[1]][[2]]
  return(B)
}


#######################################################################################
# Apply the first-significant-digit law
# Reynolds (2012) calculates the expected first-siginificant-digit distribution as follow:
# "The first-significant-digit distribution, P(n), for a LW is readily computed by noting
# that for each decade (across each decade step lengths increase by a factor of 10) there
# is the same relative probability for the various integers; thus when mu !=1,
# P(n) = int_n^{n+1} (l^{-mu}) dl =
# (1/(1-mu)) * [(n+1)^{1-mu} - n^{1-mu}]     (2)
# (neglecting a normalization factor) (Pietronero et al. 2001).This presupposes that the
# numerical value of the lower cutoff, a, is a power of 10 (1, 10, 100, . . .)."
# Reynolds (2012) then compares the observed distribution of the first-signficant-digit
# to the expected using a Pearson's chi-square test:
# "Typically the difference between the first-digit distributions for a LW with mu = 2 and
# the CCRW, as measured by Pearson's chi-squared test statistic, is statistically 
# significant (df =8, P > 0.05) when there are more than 50 steps in the walks."
# Note that if the sample size is too small the chi-square test will not be accurate
# and will return warnings

fsd <- function(SL, mu, g=FALSE){
  # Inputs are:
  #   SL is the simulated time-series of step lengths, 
  #   mu is the shape parameter of the Pareto distribution, and
  #   g specifies whether you want a graph to be produced.
  
  # Calculate the expected distribution
  # P(n) = (1/(1-mu)) * [(n+1)^{1-mu} - n^{1-mu}]
  fsdE <- 1:9
  fsdE <- (1/(1-mu))*((fsdE+1)^(1-mu) - (fsdE)^(1-mu))
  # Reynolds (2012) mentions that he is neglecting the normalising constant.
  # However, you need the expected distribution of the chi-square test to be normalised.
  fsdE <- fsdE/sum(fsdE)
  
  # Calculate the observed distribution of first-significant-digit
  # First, we use formatC to format the SL in scientific format and in string,
  # e.g., 0.045 becomes 4.5e-02 and 12.6 becomes 1.26e+1.
  # Second, we take the first number using substring.
  SL <- as.numeric(substring(formatC(SL, format="e"),1,1))
  # Use histogram to get the frequency of each first-significant-digit
  fsdO <- hist(SL, breaks=0:9, plot=FALSE)$counts
  
  # Do the chi-square test to compare the expected and observed distribution
  res <- chisq.test(fsdO, p=fsdE)
  
  # Making a graph if requested
  if(g){
    fsdO <- hist(SL, breaks=0:9, freq=FALSE, ylim=c(0,1), xaxt="n",
                 main="", xlab="", ylab= "P(n)", las=1)
    mtext("n",side=1, line=0.95, cex=0.9)
    box()
    xa <- (1:9-0.5)
    axis(1,xa,1:9)
    lines(fsdE~xa, type="o")
    mtext(substitute(paste(chi^2, " = ", cst,  ", df = ", d), 
                     list(cst=formatC(res[[1]],digits=2,format="f"),
                          d=res[[2]])),
          outer=FALSE, adj=0.95, line=-1.8, cex=0.85)
    mtext(paste("p-value = ",formatC(res[[3]],digits=2,format="f")),
          outer=FALSE, adj=0.95, line=-2.6, cex=0.85)
  }
  
  res <- cbind(chisq=res[[1]],pval=res[[3]])
  return(res)
}

#######################################################################################
# Simulations for the CCRW
# We created a set of simulations to reproduce the results of Reynolds(2012).
# We have made a few choices either because the exact values were not specified by Reynolds (2012) 
# or because we believe that the changes might be cause CCRWs to be misidentified for a LW.
# We outline some of the important parameter choices.
# 1) As explained in our comment, the Akaike weigth and MLE mu values for the LW 
# depends on the value of the cut-off in relation to minimum observed step length.
# Because the equation for the first-significant-digit is only appropriate for powers of 10
# (see p. 1231 of Reynolds (2012)), we set the cut-off value to 1. 
# To explore a large range of CCRWs, we set the mean step lengths of the intensive behavior, sI,
# slightly higher than this cut-off at 1.2, and looked at all integer values
# for the mean step length of the extensive behaviour, sE, from 2 to 100.
# 2) We investigate pI >= 0.5 as pI < 0.5 are not neceseraly good representation of a biological process.
# 3) Reynolds (2012) made some simulations with 1 000 and others with 10 000 step lengths.
# We chose to consistently use 1 000 step lengths. 
# We do 50 simmulations for each parameter combination, 
# as this is what Reynolds (2012) does for the power-spectra.
# Because all step lengths smaller than the cut-off value are removed from the analysis,
# and we wanted the simulations to be consitent in their sample size,
# the sample size represent the number of step lengths greater than the cut-off.
# Similarly, the sample size used for the power-spectra is the size of the u(t),
# not the size of movement path simulated to create that u(t). 
# To acheive this, we had to simulate much larger movement paths than the required sample size.
# 4) Although this is not clearely stated in the text, the Fig. 2 of Reynolds (2012)
# only presents the power-spectra results over the range of frequency f 1 to 500.
# To be consistent with his analysis we also only looked at these frequencies.
# 5) The number of bins used for the G-test is smaller than the one we think Reynolds (2012) used.
# But we did this to have an adequate number of expected values in each bins.
# We think that the degree of freedom presented by Reynolds (2012)
# is for his 10 000 steps simulations, not his 1000 steps simulations.
# We use 102 bins for our 1000 steps simulations
# to be analogous of using 1002 for 10 000 steps simulations.
# 6) We do not use the cut-off for the power-spectra analysis
# because Reynolds (2012) emphasizes that an advantage of the power-spectra
# is that you do not need to use a cut-off value.

# Parameters of the simulations
a <- 1 # cut-off
sI <- 1.2 # mean step length of intensive phase
sE <- 2:100 # mean step length of extensive phase
pI <- seq(0.5, 0.9, by=0.1) # probability of being in the intensive phase
n <- 1000 # Final sample size
nMP <- 3*n # Number of steps simulated
nsim <- 50 # Number of simulations
lfmid <- 500 # The number of nonzero frequencies to look at
nB <- 102 # Number of bins for the G-test

# Creating vectors and matrices that will be filled by the results
res <- cbind(n=n, a=a, lfmid=lfmid,
             sI, sE=rep(sE,each=length(pI)), pI=rep(pI,length(sE)),
             MLEmu=NA, wLW=NA, Gtest=NA,
             betaPS=NA, fsdtMLEmu=NA) # overall results
aw <- matrix(nrow=nsim, ncol=nrow(res)*2) # Akaike weight & MLE
gt <- matrix(nrow=nsim, ncol=nrow(res)) # G-test
Smatrix <- matrix(nrow=lfmid, ncol=nrow(res)) # Average spectrum
fsdtMLEmu <- matrix(nrow=nsim, ncol=nrow(res)) # fist-sig.-digit
SLmatrix <- array(dim=c(n,nsim,nrow(res))) # Actuall step length to make figures

set.seed(24)
# This loop will take 10-30 min to run.
for(i in 1:nrow(res)){
  S <- matrix(nrow=lfmid, ncol=nsim) # To average the power-spectra
  for (j in 1:nsim){
    # Simulate CCRW
    SL <- simmCCRW(nMP, res[i,6], sI, res[i,5]) # Simulate the CCRW
    S[,j] <- ps(SL, n)[1:lfmid,2] # Apply the power-spectra
    SLmatrix[,j,i] <- SL[SL >= a][1:n] # Apply the cut-off and take the specified sample size
    aw[j,i*2+c(-1,0)] <- wLW(SLmatrix[,j,i], a) # Get the Akaike weigth and MLE
    gt[j,i] <- GTest(SLmatrix[,j,i], wLW(SLmatrix[,j,i], a)[1], a, nB) # Apply the G-test
    fsdtMLEmu[j,i] <-  fsd(SLmatrix[,j,i], wLW(SLmatrix[,j,i], a)[1])[2] # Apply the first-sig.digit
  }
  Smatrix[,i] <- rowMeans(S) # Average the spectrum
  res[i, 10] <- getB(cbind(ps(SL, n)[1:lfmid,1], Smatrix[,i])) # Get the mean beta value
}
# Get mean MLE mu and mean Akaike weight
res[, 7:8] <- colMeans(aw)[c(seq(1,nrow(res)*2,by=2), seq(2,nrow(res)*2,by=2))]
# Calculate the proportion of G-tests that are sig. dif. from LW with MLE mu
res[, 9] <- colSums(gt<0.05)/nsim
# Calculate the proportion of fsdt that are sig. dif. from LW with MLE mu
res[, 11] <- colSums(fsdtMLEmu<0.05)/nsim
save.image(file="resCCRW.RData")

#######################################################################################
# Fig. 2 Exploring parameter space

# Settings for the ggplot
ggt <- theme(axis.text  = element_text(size = 13 , colour = "black"),
             axis.title = element_text(size = 14, face="plain"),
             legend.key.height = unit(0.6, "cm"),
             legend.key.width = unit(0.5, "cm"),
             legend.text = element_text(size = 12 , colour = "black"),
             legend.title = element_text(size = 12, face="plain"),
             legend.title.align = 1,
             plot.title = element_text(size = 12, hjust=0),
             plot.margin= unit(c(0,-0.5,0,0), "lines"))
ggsx <- scale_x_continuous(expand = c(0,0))
ggsy <- scale_y_continuous(expand = c(0,0))


# Power-spectra
pPS <- ggplot(data.frame(res), aes(sE, pI, fill = betaPS)) + geom_tile() +
  xlab(expression(s[E])) + 
  ylab(expression(p[I])) +
  ggt +
  scale_fill_gradient(limits = c(0, max(res[,"betaPS"])),
                      low = "black", high = "white", 
                      name=  expression(beta))  +
  ggsx + ggsy +
  ggtitle("D) Power-spectra")

# First-significant-digit
pFD <- ggplot(data.frame(res), aes(sE, pI, fill = 100*fsdtMLEmu)) + geom_tile() +
  xlab(expression(s[E])) + 
  ylab(expression(p[I])) +
  ggt +
  scale_fill_gradient(limits = c(0, 100), low = "white", high = "black", 
                      name= "% \n sig. dif.") +
  ggsx + ggsy +
  ggtitle("E) First-siginificant-digit")

# MLE mu
pMLEmu <- ggplot(data.frame(res), aes(sE, pI, fill = MLEmu)) + geom_tile() +
  xlab(expression(s[E])) + 
  ylab(expression(p[I])) +
  ggt +
  scale_fill_gradient(low = "black", high = "white", 
                      name= expression(mu)) +
   geom_point(aes(colour = !(MLEmu >= 1.95 & MLEmu<=2.05)), size=0.5) +
   scale_colour_manual(values=c("white", NA), guide="none") +
  ggsx + ggsy +
  ggtitle("A) MLE mu")

# Akaike weigth for the LW
pwLW <- ggplot(data.frame(res), aes(sE, pI, fill = wLW)) + geom_tile() +
  xlab(expression(s[E])) + 
  ylab(expression(p[I])) +
  ggt +
  scale_fill_gradient(low = "black", high = "white", 
                      name= expression(w[LW])) +
  ggsx + ggsy +
  ggtitle("B) Akaike weigth for LW")

# G-test
pGT <- ggplot(data.frame(res), aes(sE, pI, fill = 100*Gtest)) + geom_tile() +
  xlab(expression(s[E])) + 
  ylab(expression(p[I])) +
  ggt +
  scale_fill_gradient(limits = c(0, 100), low = "white", high = "black", 
                      name= "% \n sig. dif.") +
  ggsx + ggsy +
  ggtitle("C) G-test")

windows(width=9, height=4.5)
par(mar=c(1,4,3.5,0.2))
grid.arrange(pMLEmu, pwLW, pGT, pPS, pFD, ncol=3)
savePlot("Fig2","pdf")
savePlot("Fig2","jpg")

#######################################################################################
# Fig. 1
windows(width=9, height=5)
split.screen(c(2,4), erase=TRUE)
parS <- list(cex.axis=0.9, cex.lab=0.9, las=1, tcl=-0.2, mar=c(2,2.8,2.1,0.2), mgp=c(1.8,0.3,0))

f <- (((2:n)-1)/n )[1:lfmid] # Frequencies used in all figures
# Fomating plot and mtext for the power-spectra plots
psf <- function(S, f, mt, bb, mler, wr, ylim){
  par(parS)
  plot(log(S,10)~log(f,10), type="l", xaxt="n", yaxt="n", ylab ="S(f)",xlab="", ylim=log(ylim,10))
  mtext("f",side=1, line=0.9, cex=0.9)
  
  # Axis (log-log) with format 10^k
  ten2k <- function(k){substitute(10^k, list(k=formatC(k,width=-3)))}
  SeqLog <- seq(floor(log(ylim[1],10)),ceiling(log(ylim[2],10)),by=0.5)
  axis(2,at=SeqLog, labels=do.call("expression",lapply(SeqLog, ten2k)))
  SeqLog <- seq(floor(log(min(f),10)),ceiling(log(max(f),10)),by=1)
  axis(1,at=SeqLog, labels=do.call("expression",lapply(SeqLog, ten2k)))
    
  # Extra info added in the figure
  mtext(substitute(paste(beta," = ", bb), list(bb=formatC(bb,digits=2,format="f"))),
        outer=FALSE, adj=0.95, line=-1.6, cex=0.85)
  mtext(mt, adj=0, line=1, cex=0.9)
  mtext(substitute(paste("(", bar(mu)[MLE], " = ", mmle, ", ", bar(w), " = ", mw, ")"), 
                   list(mmle = formatC(mler,digits=2,format="f"),
                        mw = formatC(wr,digits=2,format="f"))),
        adj=0, line=0, cex=0.85)
}

# Formating of part of the text for the fsd
fsdf <- function(mt, mler, wr){
  mtext(mt, adj=0, line=1, cex=0.9)
  mtext(substitute(paste("(", mu[MLE], " = ", mmle, ", w = ", mw, ")"), 
                   list(mmle = formatC(mler,digits=2,format="f"),
                        mw = formatC(wr,digits=2,format="f"))),
        adj=0, line= 0, cex=0.85)
}

##############################
# Extra simulations
set.seed(79)
# Some parameters stay the same for all simulation sets:
# e.g., nsim = 50, lfmid = 500, n = 1000

# For panels A & E: DTLW with mu=2, a=10, xmin=1, xmax=100000
SLDTLW <- matrix(nrow=n, ncol=nsim)
psDTLW <- matrix(nrow=n-1, ncol=nsim)
mleDTLW <- matrix(nrow=2, ncol=nsim)

for(i in 1:nsim){
  DTLW <- simmDTLW(n*30, 2, 1, 100000)
  psDTLW[ ,i] <- ps(DTLW,n)[,2]
  SLDTLW[,i] <- DTLW[DTLW>=10][1:n]
  mleDTLW[ ,i] <- wLW(SLDTLW[,i], 10)
}
psDTLWmid <- rowMeans(psDTLW)[1:lfmid]

# Panels B & F: CCRW with Reynolds (2012) parameters
# sI = 15, sE =74, pI= 0.9, a=10
SLCCRWR <- matrix(nrow=n,ncol=nsim)
psCCRWR <- matrix(nrow=n-1,ncol=nsim)
mleCCRWR <- matrix(nrow=2,ncol=nsim)
for(i in 1:nsim){
  CCRWR <- simmCCRW(n*3, 0.9, 15, 74)
  psCCRWR[ ,i] <- ps(CCRWR,n)[,2]
  SLCCRWR[ ,i] <- CCRWR[CCRWR>=10][1:n]
  mleCCRWR[ ,i] <- wLW(SLCCRWR[,i],10)
}
psCCRWRmid <- rowMeans(psCCRWR)[1:lfmid]

# Panels C & G: TLW with mu=2, a=1, xmin=1, xmax=100000
SLTLW <- matrix(nrow=n, ncol=nsim)
psTLW <- matrix(nrow=n-1, ncol=nsim)
mleTLW <- matrix(nrow=2, ncol=nsim)

for(i in 1:nsim){
  TLW <- simmTLW(n*30, 2, 1, 100000)
  psTLW[ ,i] <- ps(TLW,n)[,2]
  SLTLW[ ,i] <- TLW[TLW>=1][1:n]
  mleTLW[ ,i] <- wLW(SLTLW[,i],1)
}
psTLWmid <- rowMeans(psTLW)[1:lfmid]

###############################
# Index for simmulations sets
# Pannels D & H: CCRW that would be confused for a TLW with mu =2
# sI = 1.2, sE =12, pI= 0.9, a=1

# From Reynolds (2012):
# "[...] CCRW were misidentified as being TLW with mu ~= 2 because 
# wp > 0.99 and because the maximum likelihood estimates for mu were
# between 1.95 and 2.05."
# So we look at the simulation that are misidenitified for TLW
# Find the paths that have mu 1.95 and 2.05 & wLW > 0.99
mu2wLW1 <- res[,"MLEmu"] > 1.95 & res[,"MLEmu"] < 2.05 & res[,"wLW"] > 0.99
cbind(which(mu2wLW1), res[mu2wLW1,])

# This gives us a good example of a CCRW that would be misidentify for LW with mu 2
k <- 55
res[k,]

#################################
# To standardize the yaxis of the power-spectra
ylimps <- c(min(psDTLWmid, psCCRWRmid, psTLWmid, Smatrix[1:lfmid,k]),
            max(psDTLWmid, psCCRWRmid, psTLWmid, Smatrix[1:lfmid,k]))


###############################
# Panels A & E: DTLW with mu=2, a=10, xmin=1, xmax=100000

##
# Power-spectra
screen(1)
psf(psDTLWmid, f, "A) Discrete TLW", getB(cbind(f,psDTLWmid)),
    mean(mleDTLW[1,]), mean(mleDTLW[2,]), ylimps)

##
# First significant digits
screen(5)
par(parS)
SL <- SLDTLW[,which(mleDTLW[1,]>1.95 & mleDTLW[1,]<2.05 & mleDTLW[2,]>0.99)[1]]   
fsd(SL, 2, g=TRUE) # Not sig. dif
fsdf("E) Discrete TLW", wLW(SL, 10)[1], wLW(SL,10)[2])

###############################
# Panels B & F: CCRW with Reynolds (2012) parameters
# sI = 15, sE =74, pI= 0.9, a=10

##
# Power-spectra
screen(2)
psf(psCCRWRmid,f, "B) Reynolds' (2012) CCRW", getB(cbind(f,psCCRWRmid)),
    mean(mleCCRWR[1,]), mean(mleCCRWR[2,]), ylimps)

##
# First significant digits
screen(6)
par(parS)
SL <- SLCCRWR[,which(mleCCRWR[1,] > 1.95 & mleCCRWR[1,] < 2.05 & mleCCRWR[2,]>0.99)[1]] 
fsd(SL,2,g=TRUE) # Sig. diff from LW
fsdf("F) Reynolds' (2012) CCRW", wLW(SL,10)[1], wLW(SL, 10)[2])

###############################
# Pannels C & G: TLW with mu=2, a=1, xmin=1, xmax=100000

##
# Power-spectra
screen(3)
psf(psTLWmid, f, "C) Continuous TLW", getB(cbind(f,psTLWmid)),
    mean(mleTLW[1,]), mean(mleTLW[2,]), ylimps)

##
# First-significant digits
screen(7)
par(parS)
SL <- SLTLW[,which(mleTLW[1,]>1.95 & mleTLW[1,]<2.05 & mleTLW[2,]>0.99)[1]]
fsd(SL,2,g=TRUE) # Not sig. dif
fsdf("G) Continuous TLW", wLW(SL,1)[1], wLW(SL,1)[2])

###############################
# Pannels D & H: CCRW that would be confused for a TLW with mu =2
# sI = 1.2, sE =, pI= 0.9, a=1

##
# Power spectra
screen(4)
psf(Smatrix[1:lfmid,k],f, "D) Our CCRW", res[k,"betaPS"],
    res[k,"MLEmu"], res[k,"wLW"], ylimps)

##
# First significant digits
screen(8)
par(parS)
SL <- SLmatrix[,which(aw[,(2*k-1)]>1.95 & aw[,(2*k-1)]<2.05 & aw[,(2*k)]>0.99)[1],k]
fsd(SL,2,g=TRUE) # Sig. dif
fsdf("H) Our CCRW", wLW(SL, 1)[1], wLW(SL, 1)[2])

close.screen(all=TRUE)
savePlot("Fig1","pdf")
savePlot("Fig1","jpg")

#######################################################################################
# Fig 3. Displaying the impact of cut-off on the evidence for LW with mu=2
# Reynolds parameters for CCRW: 1000 steps, sI=15, sE=74, pI=0.9, pE=0.1, xmin=0, a=10
nSim <- 100 # instead of the 50 used before, to make nicer y-scale

set.seed(71)
# To save results
wResReyPar <- matrix(nrow=nSim, ncol=4)
colnames(wResReyPar) <- c("mleMUa10", "wLWa10", "mleMUamin", "wLWamin")
amSL <- numeric(nSim)
for(i in 1:nSim){
  SL <- simmCCRW(3*n, 0.9, 15, 74)
  # Reynolds parameters with cut-off 10
  wResReyPar[i,1:2] <- wLW(SL[SL>=10][1:n], 10)
  # Reynolds parameters with cut-off as minimum observed step length
  wResReyPar[i,3:4] <- wLW(SL[1:n], min(SL[1:n]))
  amSL[i] <- min(SL[1:n])
}

mean(amSL)

windows(width=9, height=5)
layout(matrix(1:2,nrow=1))
hist(wResReyPar[,2], xlim=c(0,1), ylim=c(0,100), breaks=seq(0,1,by=0.1),
     xlab="w", ylab="Number of tracks", main="")
title(main="A) Cut-off = 10",adj=0)
mtext(substitute(paste("(",bar(mu)[MLE], " =" , ll, ")"), 
                 list(ll=format(mean(wResReyPar[,1]),digits=2))), adj=0)
box()      

hist(wResReyPar[,4], xlim=c(0,1), ylim=c(0,100), breaks=seq(0,1,by=0.1),
     xlab="w", ylab="Number of tracks", main="")
title(main="B) Cut-off = min. observed step length",adj=0)
mtext(substitute(paste("(",bar(mu)[MLE], " =" , ll, ")"), 
                 list(ll=format(mean(wResReyPar[,3]),digits=2))), adj=0)
box()

savePlot("Fig3", "pdf")
savePlot("Fig3", "jpg")

# References: 
# Sokal, P.R., and F.J. Rohlf (1995). Biometry: the principles and practice of 
#   statistics in biological research. 3rd Ed. W.H. Freeman, New York, USA.
# Reynolds, A.M (2012). Distinguishing between Levy walks and strong 
#   alternative models. Ecology 83: 1228-1233